@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1998--2019  The R Core Team.
+ *  Copyright (C) 1998--2021  The R Core Team.
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -80,6 +80,11 @@ Rcomplex Rf_ComplexFromReal(double, int*);
 #include <Rinternals.h>		/*-> Arith.h, Boolean.h, Complex.h, Error.h,
 				  Memory.h, PrtUtil.h, Utils.h */
 #undef CALLED_FROM_DEFN_H
+
+const char * Rf_translateCharFP(SEXP);
+const char * Rf_translateCharFP2(SEXP);
+const char * Rf_trCharUTF8(SEXP);
+
 extern0 SEXP	R_CommentSymbol;    /* "comment" */
 extern0 SEXP	R_DotEnvSymbol;     /* ".Environment" */
 extern0 SEXP	R_ExactSymbol;	    /* "exact" */
@@ -249,6 +254,10 @@ extern char *strdup(const char *s1);
 extern int strncasecmp(const char *s1, const char *s2, size_t n);
 #endif
 
+/* safer alternative */
+extern char *Rstrdup(const char *s);
+
+
 /* Glibc manages to not define this in -pedantic -ansi */
 #if defined(HAVE_PUTENV) && !defined(putenv) && defined(HAVE_DECL_PUTENV) && !HAVE_DECL_PUTENV
 extern int putenv(char *string);
@@ -508,6 +517,7 @@ Rboolean (NO_SPECIAL_SYMBOLS)(SEXP b);
 */
 typedef struct {
     int tag;
+    int flags;
     union {
 	int ival;
 	double dval;
@@ -517,6 +527,7 @@ typedef struct {
 # define PARTIALSXP_MASK (~255)
 # define IS_PARTIAL_SXP_TAG(x) ((x) & PARTIALSXP_MASK)
 # define RAWMEM_TAG 254
+# define CACHESZ_TAG 253
 
 #ifdef R_USE_SIGNALS
 /* Stack entry for pending promises */
@@ -550,6 +561,7 @@ typedef struct RCNTXT {
     SEXP restartstack;          /* stack of available restarts */
     struct RPRSTACK *prstack;   /* stack of pending promises */
     R_bcstack_t *nodestack;
+    R_bcstack_t *bcprottop;
     SEXP srcref;	        /* The source line in effect */
     int browserfinish;          /* should browser finish this context without
                                    stopping */
@@ -709,10 +721,13 @@ extern0 Rboolean R_CBoundsCheck	INI_as(FALSE);	/* options(CBoundsCheck) */
 extern0 MATPROD_TYPE R_Matprod	INI_as(MATPROD_DEFAULT);  /* options(matprod) */
 extern0 int	R_WarnLength	INI_as(1000);	/* Error/warning max length */
 extern0 int	R_nwarnings	INI_as(50);
+
+/* C stack checking */
 extern uintptr_t R_CStackLimit	INI_as((uintptr_t)-1);	/* C stack limit */
 extern uintptr_t R_OldCStackLimit INI_as((uintptr_t)0); /* Old value while
 							   handling overflow */
 extern uintptr_t R_CStackStart	INI_as((uintptr_t)-1);	/* Initial stack address */
+/* Default here is for Windows: set from configure in src/unix/system.c */
 extern int	R_CStackDir	INI_as(1);	/* C stack direction */
 
 #ifdef R_USE_SIGNALS
@@ -722,7 +737,7 @@ extern0 struct RPRSTACK *R_PendingPromises INI_as(NULL); /* Pending promise stac
 /* File Input/Output */
 LibExtern Rboolean R_Interactive INI_as(TRUE);	/* TRUE during interactive use*/
 extern0 Rboolean R_Quiet	INI_as(FALSE);	/* Be as quiet as possible */
-extern Rboolean  R_Slave	INI_as(FALSE);	/* Run as a slave process */
+extern Rboolean  R_NoEcho	INI_as(FALSE);	/* do not echo R code */
 extern0 Rboolean R_Verbose	INI_as(FALSE);	/* Be verbose */
 /* extern int	R_Console; */	    /* Console active flag */
 /* IoBuffer R_ConsoleIob; : --> ./IOStuff.h */
@@ -772,8 +787,10 @@ extern0 int	R_NShowCalls INI_as(50);
 LibExtern Rboolean utf8locale  INI_as(FALSE);  /* is this a UTF-8 locale? */
 LibExtern Rboolean mbcslocale  INI_as(FALSE);  /* is this a MBCS locale? */
 extern0   Rboolean latin1locale INI_as(FALSE); /* is this a Latin-1 locale? */
+LibExtern int      R_MB_CUR_MAX INI_as(FALSE); /* corrected variant of MB_CUR_MAX */
 #ifdef Win32
 LibExtern unsigned int localeCP  INI_as(1252); /* the locale's codepage */
+LibExtern unsigned int systemCP  INI_as(437);  /* the ANSI codepage, GetACP */
 extern0   Rboolean WinUTF8out  INI_as(FALSE);  /* Use UTF-8 for output */
 extern0   void WinCheckUTF8(void);
 #endif
@@ -802,6 +819,7 @@ void resetTimeLimits(void);
 #define R_BCNODESTACKSIZE 200000
 LibExtern R_bcstack_t *R_BCNodeStackTop, *R_BCNodeStackEnd;
 extern0 R_bcstack_t *R_BCNodeStackBase;
+extern0 R_bcstack_t *R_BCProtTop;
 extern0 int R_jit_enabled INI_as(0); /* has to be 0 during R startup */
 extern0 int R_compile_pkgs INI_as(0);
 extern0 int R_check_constants INI_as(0);
@@ -815,6 +833,8 @@ extern SEXP R_findBCInterpreterSrcref(RCNTXT*);
 extern SEXP R_getCurrentSrcref();
 extern SEXP R_getBCInterpreterExpression();
 
+void R_BCProtReset(R_bcstack_t *);
+
 LibExtern int R_num_math_threads INI_as(1);
 LibExtern int R_max_num_math_threads INI_as(1);
 
@@ -856,7 +876,11 @@ LibExtern SEXP R_LogicalNAValue INI_as(NULL);
 
 /* for PCRE as from R 3.4.0 */
 extern0 Rboolean R_PCRE_use_JIT INI_as(TRUE);
+#ifdef HAVE_PCRE2
+extern0 int R_PCRE_study INI_as(-2);
+#else
 extern0 int R_PCRE_study INI_as(10);
+#endif
 extern0 int R_PCRE_limit_recursion;
 
 
@@ -941,7 +965,7 @@ extern0 int R_PCRE_limit_recursion;
 # define IntegerFromString	Rf_IntegerFromString
 # define internalTypeCheck	Rf_internalTypeCheck
 # define isValidName		Rf_isValidName
-# define installTrChar		Rf_installTrChar
+//# define installTrChar		Rf_installTrChar
 # define ItemName		Rf_ItemName
 # define jump_to_toplevel	Rf_jump_to_toplevel
 # define KillAllDevices		Rf_KillAllDevices
@@ -956,7 +980,7 @@ extern0 int R_PCRE_limit_recursion;
 # define mat2indsub		Rf_mat2indsub
 # define matchArg		Rf_matchArg
 # define matchArgExact		Rf_matchArgExact
-# define matchArgs		Rf_matchArgs
+# define matchArgs_NR		Rf_matchArgs_NR
 # define matchArgs_RC		Rf_matchArgs_RC
 # define matchPar		Rf_matchPar
 # define Mbrtowc		Rf_mbrtowc
@@ -1006,6 +1030,9 @@ extern0 int R_PCRE_limit_recursion;
 # define strmat2intmat		Rf_strmat2intmat
 # define substituteList		Rf_substituteList
 # define TimeToSeed		Rf_TimeToSeed
+# define translateCharFP	Rf_translateCharFP
+# define translateCharFP2	Rf_translateCharFP2
+# define trCharUTF8      	Rf_trCharUTF8
 # define tspgets		Rf_tspgets
 # define type2symbol		Rf_type2symbol
 # define unbindVar		Rf_unbindVar
@@ -1075,7 +1102,7 @@ void R_SetVarLocValue(R_varloc_t, SEXP);
 #define KEEPNA			64
 #define S_COMPAT       		128
 #define HEXNUMERIC             	256
-#define DIGITS16             	512
+#define DIGITS17		512
 #define NICE_NAMES             	1024
 /* common combinations of the above */
 #define SIMPLEDEPARSE		0
@@ -1191,7 +1218,7 @@ SEXP markKnown(const char *, SEXP);
 SEXP mat2indsub(SEXP, SEXP, SEXP);
 SEXP matchArg(SEXP, SEXP*);
 SEXP matchArgExact(SEXP, SEXP*);
-SEXP matchArgs(SEXP, SEXP, SEXP);
+SEXP matchArgs_NR(SEXP, SEXP, SEXP);
 SEXP matchArgs_RC(SEXP, SEXP, SEXP);
 SEXP matchPar(const char *, SEXP*);
 void memtrace_report(void *, void *);
@@ -1279,6 +1306,8 @@ SEXP R_syscall(int,RCNTXT*);
 int R_sysparent(int,RCNTXT*);
 SEXP R_sysframe(int,RCNTXT*);
 SEXP R_sysfunction(int,RCNTXT*);
+RCNTXT *R_findExecContext(RCNTXT *, SEXP);
+RCNTXT *R_findParentContext(RCNTXT *, int);
 
 void R_run_onexits(RCNTXT *);
 void NORET R_jumpctxt(RCNTXT *, int, SEXP);
@@ -1291,7 +1320,8 @@ SEXP ItemName(SEXP, R_xlen_t);
 void NORET errorcall_cpy(SEXP, const char *, ...);
 void NORET ErrorMessage(SEXP, int, ...);
 void WarningMessage(SEXP, R_WARNING, ...);
-SEXP R_GetTraceback(int);
+SEXP R_GetTraceback(int);    // including deparse()ing
+SEXP R_GetTracebackOnly(int);// no        deparse()ing
 
 R_size_t R_GetMaxVSize(void);
 void R_SetMaxVSize(R_size_t);
@@ -1300,6 +1330,10 @@ void R_SetMaxNSize(R_size_t);
 R_size_t R_Decode2Long(char *p, int *ierr);
 void R_SetPPSize(R_size_t);
 
+void R_expand_binding_value(SEXP);
+
+void R_args_enable_refcnt(SEXP);
+
 /* ../main/devices.c, used in memory.c, gnuwin32/extra.c */
 #define R_MaxDevices 64
 
@@ -1317,6 +1351,8 @@ const char *EncodeString(SEXP, int, int, Rprt_adj);
 const char *EncodeReal2(double, int, int, int);
 const char *EncodeChar(SEXP);
 
+/* main/raw.c */
+int mbrtoint(int *w, const char *s);
 
 /* main/sort.c */
 void orderVector1(int *indx, int n, SEXP key, Rboolean nalast,
@@ -1338,12 +1374,13 @@ int utf8clen(char c);
 int Rf_AdobeSymbol2ucs2(int n);
 double R_strtod5(const char *str, char **endptr, char dec,
 		 Rboolean NA, int exact);
+SEXP R_listCompact(SEXP s, Rboolean keep_initial);
 
-typedef unsigned short ucs2_t;
-size_t mbcsToUcs2(const char *in, ucs2_t *out, int nout, int enc);
+typedef unsigned short R_ucs2_t;
+size_t mbcsToUcs2(const char *in, R_ucs2_t *out, int nout, int enc);
 /* size_t mbcsMblen(char *in);
-size_t ucs2ToMbcs(ucs2_t *in, char *out);
-size_t ucs2Mblen(ucs2_t *in); */
+size_t ucs2ToMbcs(R_ucs2_t *in, char *out);
+size_t ucs2Mblen(R_ucs2_t *in); */
 size_t utf8toucs(wchar_t *wc, const char *s);
 size_t utf8towcs(wchar_t *wc, const char *s, size_t n);
 size_t ucstomb(char *s, const unsigned int wc);
@@ -1362,6 +1399,8 @@ char *mbcsTruncateToValid(char *s);
 Rboolean utf8Valid(const char *str);
 char *Rf_strchr(const char *s, int c);
 char *Rf_strrchr(const char *s, int c);
+int Rvsnprintf_mbcs(char *buf, size_t size, const char *format, va_list ap);
+int Rsnprintf_mbcs(char *str, size_t size, const char *format, ...);
 
 SEXP fixup_NaRm(SEXP args); /* summary.c */
 void invalidate_cached_recodings(void);  /* from sysutils.c */
