@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2004-2015   The R Core Team.
+ *  Copyright (C) 2004-2017   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 
 #include "grDevices.h"
 #include <R_ext/Rdynload.h>
+#include <R_ext/Visibility.h>
 
 #ifndef _WIN32
 /* This really belongs with the X11 module, but it is about devices */
@@ -71,6 +72,8 @@ static const R_CallMethodDef CallEntries[] = {
     CALLDEF(palette, 1),
     CALLDEF(palette2, 1),
     CALLDEF(cairoVersion, 0),
+    CALLDEF(pangoVersion, 0),
+    CALLDEF(cairoFT, 0),
     CALLDEF(bmVersion, 0),
 
 #ifndef _WIN32
@@ -90,7 +93,7 @@ static const R_ExternalMethodDef ExtEntries[] = {
     EXTDEF(PostScript, 19),
     EXTDEF(XFig, 14),
     EXTDEF(PDF, 20),
-    EXTDEF(devCairo, 11),
+    EXTDEF(devCairo, 12),
     EXTDEF(devcap, 0),
     EXTDEF(devcapture, 1),
     EXTDEF(devcontrol, 1),
@@ -109,6 +112,9 @@ static const R_ExternalMethodDef ExtEntries[] = {
     EXTDEF(getGraphicsEvent, 1),
     EXTDEF(getGraphicsEventEnv, 1),
     EXTDEF(setGraphicsEventEnv, 2),
+    EXTDEF(setPattern, 1),
+    EXTDEF(setClipPath, 2),
+    EXTDEF(setMask, 2),
     EXTDEF(devAskNewPage, 1),
 
 #ifdef _WIN32
@@ -117,7 +123,7 @@ static const R_ExternalMethodDef ExtEntries[] = {
 #else
     EXTDEF(savePlot, 3),
     EXTDEF(Quartz, 11),
-    EXTDEF(X11, 17),
+    EXTDEF(X11, 18),
 #endif
     {NULL, NULL, 0}
 };
@@ -127,7 +133,7 @@ extern void setup_RdotApp(void);
 extern Rboolean useaqua;
 #endif
 
-void R_init_grDevices(DllInfo *dll)
+void attribute_visible R_init_grDevices(DllInfo *dll)
 {
     initPalette();
     R_registerRoutines(dll, NULL, CallEntries, NULL, ExtEntries);
