@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2001-2019   The R Core Team.
+ *  Copyright (C) 2001-2021   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -301,7 +301,9 @@ SEXP R_quick_method_check(SEXP args, SEXP mlist, SEXP fdef)
 	    /* not observed during tests, but promises in principle could come
 	       from DispatchOrEval/R_possible_dispatch */
 	    object = eval(object, Methods_Namespace);
+	PROTECT(object);
 	class = CHAR(STRING_ELT(R_data_class(object, TRUE), 0));
+	UNPROTECT(1); /* object */
 	value = R_element_named(methods, class);
 	if(isNull(value) || isFunction(value)){
 	    retValue = value;
@@ -365,8 +367,11 @@ SEXP R_quick_dispatch(SEXP args, SEXP genericEnv, SEXP fdef)
 	    object = eval(object, Methods_Namespace);
 	if(object == R_MissingArg)
 	    class = "missing";
-	else
+	else {
+	    PROTECT(object);
 	    class = CHAR(STRING_ELT(R_data_class(object, TRUE), 0));
+	    UNPROTECT(1); /* object */
+	}
 	if(ptr - buf + strlen(class) + 2 > NBUF) {
 	    UNPROTECT(1); /* mtable */
 	    return R_NilValue;
@@ -981,10 +986,16 @@ SEXP R_getClassFromCache(SEXP class, SEXP table)
 
 static SEXP do_inherited_table(SEXP class_objs, SEXP fdef, SEXP mtable, SEXP ev)
 {
-    static SEXP dotFind = NULL, f; SEXP  e, ee;
-    if(dotFind == NULL) {
-	dotFind = install(".InheritForDispatch");
+    static SEXP f = NULL;
+    SEXP e, ee;
+
+    if(f == NULL) {
+	SEXP dotFind = install(".InheritForDispatch");
+	/* NOTE: the call to findFun can lead to recursive (but it seems only one)
+	   invocation of do_inherited_table(), and hence this initialization
+	   (PR#17923). */
 	f = findFun(dotFind, R_MethodsNamespace);
+	R_PreserveObject(f);
     }
     PROTECT(e = allocVector(LANGSXP, 4));
     SETCAR(e, f); ee = CDR(e);
