@@ -1,7 +1,7 @@
 #  File src/library/tools/R/Rd2txt.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2016 The R Core Team
+#  Copyright (C) 1995-2021 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -25,8 +25,7 @@
 
 tabExpand <- function(x) {
     srcref <- attr(x, "srcref")
-    if (is.null(srcref)) start <- 0L
-    else start <- srcref[5L] - 1L
+    start <- if(is.null(srcref)) 0L else srcref[5L] - 1L
     .Call(C_doTabExpand, x, start)
 }
 
@@ -56,8 +55,9 @@ Rd2txt_options <- local({
 
 transformMethod <- function(i, blocks, Rdfile) {
     editblock <- function(block, newtext)
-    	list(structure(newtext, Rd_tag = attr(block, "Rd_tag"),
-    	                   srcref = attr(block, "srcref")))
+    	list(tagged(newtext,
+                    attr(block, "Rd_tag"),
+                    attr(block, "srcref")))
 
     # Most of the internal functions below are more like macros
     # than functions; they mess around with these variables:
@@ -226,7 +226,7 @@ transformMethod <- function(i, blocks, Rdfile) {
 	findClose(j)
 	chars[char] <- ""
 	blocks[j] <- editblock(blocks[[j]],
-	                         paste(chars, collapse=""))
+                               paste(chars, collapse=""))
 
 	methodtype <- ""
     } else {
@@ -249,32 +249,36 @@ transformMethod <- function(i, blocks, Rdfile) {
             }
             cl <- paste("##", cl, collapse="\n")
             c( blocks[seq_len(i-1L)],
-              list(structure(paste0("## S4 ", methodtype, "method for signature \n"),
-                             Rd_tag="RCODE", srcref=srcref)),
-              list(structure(cl, Rd_tag="TEXT", srcref=srcref)),
-              list(structure("\n", Rd_tag="RCODE", srcref=srcref)),
+              list(tagged(paste0("## S4 ", methodtype,
+                                 "method for signature \n"),
+                          "RCODE", srcref)),
+              list(tagged(cl, "TEXT", srcref)),
+              list(tagged("\n", "RCODE", srcref)),
               blocks[-seq_len(i)] )
         } else
             c( blocks[seq_len(i-1L)],
-              list(structure(paste0("## S4 ", methodtype, "method for signature '"),
-                             Rd_tag="RCODE", srcref=srcref)),
+              list(tagged(paste0("## S4 ", methodtype,
+                                 "method for signature '"),
+                          "RCODE", srcref)),
               class,
-              list(structure("'\n", Rd_tag="RCODE", srcref=srcref)),
+              list(tagged("'\n", "RCODE", srcref)),
               blocks[-seq_len(i)] )
     } else if (default)
     	blocks <- c( blocks[seq_len(i-1)],
-    		     list(structure(paste0("## Default S3 ", methodtype, "method:\n"),
-    		     	       Rd_tag="RCODE", srcref=srcref)),
+                     list(tagged(paste0("## Default S3 ", methodtype,
+                                        "method:\n"),
+                                 "RCODE", srcref)),
     		     blocks[-seq_len(i)] )
     else
     	blocks <- c( blocks[seq_len(i-1)],
-		     list(structure(paste0("## S3 ", methodtype, "method for class '"),
-			   Rd_tag="RCODE", srcref=srcref)),
+                     list(tagged(paste0("## S3 ", methodtype,
+                                        "method for class '"),
+                                 "RCODE", srcref)),
 		     class,
-		     list(structure("'\n", Rd_tag="RCODE", srcref=srcref)),
+		     list(tagged("'\n", "RCODE", srcref)),
 		     blocks[-seq_len(i)] )
     blocks
-}
+}# transformMethod()
 
 Rd2txt <-
     function(Rd, out="", package = "", defines=.Platform$OS.type,
@@ -375,7 +379,7 @@ Rd2txt <-
 
     put <- function(...) {
         txt <- paste0(..., collapse="")
-        trail <- grepl("\n$", txt)
+        trail <- endsWith(txt, "\n")
         # Convert newlines
         txt <- strsplit(txt, "\n", fixed = TRUE)[[1L]]
         if (dropBlank) {
@@ -453,7 +457,7 @@ Rd2txt <-
          ((li$codepage >= 1250 && li$codepage <= 1258) || li$codepage == 874)) ||
         li[["UTF-8"]]
 
-    if(!identical(getOption("useFancyQuotes"), FALSE) &&
+    if(!isFALSE(getOption("useFancyQuotes")) &&
        use_fancy_quotes) {
         ## On Windows, Unicode literals are translated to local code page
     	LSQM <- intToUtf8("0x2018") # Left single quote
@@ -491,8 +495,6 @@ Rd2txt <-
         txt <- as.character(x)
         if(inEqn) txt <- txt_eqn(txt)
         txt <- fsub('"\\{"', '"{"', txt)
-        ## \dots gets left in noquote.Rd
-        txt <- fsub("\\dots",  "...", txt)
         put(txt)
     }
 
@@ -597,28 +599,16 @@ Rd2txt <-
                    else writeContent(block,tag)
                },
                "\\email" = {
-                   put("<email: ",
-                       trimws(gsub("\n", "",
-                                   paste(as.character(block),
-                                         collapse=""))),
-                       ">")
+                   put("<email: ", lines2str(as.character(block)), ">")
                },
                "\\url" = {
-                   put("<URL: ",
-                       trimws(gsub("\n", "",
-                                   paste(as.character(block),
-                                         collapse=""))),
-                       ">")
+                   put("<URL: ", lines2str(as.character(block)), ">")
                },
                "\\href" = {
                    opts <- Rd2txt_options()
                    writeContent(block[[2L]], tag)
                    if (opts$showURLs)
-  			put(" (URL: ",
-  			    trimws(gsub("\n", "",
-                                        paste(as.character(block[[1L]]),
-                                              collapse=""))),
-  			    ")")
+  			put(" (URL: ", lines2str(as.character(block[[1L]])), ")")
                },
                "\\Sexpr"= put(as.character.Rd(block, deparse=TRUE)),
                "\\acronym" =,
@@ -641,8 +631,10 @@ Rd2txt <-
                "\\sQuote" =,
                "\\dQuote"= writeQ(block, tag) ,
                "\\preformatted"= {
-                   putf("\n")
+                   blankLine()
+                   wrap(FALSE)
                    writeCodeBlock(block, tag)
+                   blankLine()
                },
                "\\verb"= put(block),
                "\\linkS4class" =,
@@ -662,7 +654,7 @@ Rd2txt <-
                    ## Test to see if we can convert the encoded version
                    txt <- as.character(block[[1L]])
                    test <- iconv(txt, "UTF-8", outputEncoding, mark = FALSE)
-                   txt <- if(!is.na(test)) txt else as.character(block[[2L]])
+                   txt <- if(!anyNA(test)) txt else as.character(block[[2L]])
                    put(txt)
                } ,
                "\\eqn" = {
@@ -755,6 +747,7 @@ Rd2txt <-
                         else
                             entries
                     })
+        if(!length(entries)) return()
         rows <- entries[[length(entries)]]$row
         cols <- max(sapply(entries, function(e) e$col))
         widths <- rep_len(0L, cols)
