@@ -1,7 +1,7 @@
 #  File src/library/tools/R/dynamicHelp.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2016 The R Core Team
+#  Copyright (C) 1995-2019 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -27,8 +27,9 @@
 ##             or by file, /library/<pkg>/html/<file>.html
 httpd <- function(path, query, ...)
 {
-    .HTMLdirListing <- function(dir, base, up)
-    {
+    linksToTopics <-
+        config_val_to_logical(Sys.getenv("_R_HELP_LINKS_TO_TOPICS_", "TRUE"))
+    .HTMLdirListing <- function(dir, base, up) {
         files <- list.files(dir)    # note, no hidden files are listed
         out <- HTMLheader(paste0("Listing of directory<br/>", dir),
         		  headerTitle = paste("R:", dir), logo=FALSE,
@@ -41,12 +42,11 @@ httpd <- function(path, query, ...)
                      paste0("<dd>", mono(iconv(urls, "", "UTF-8")), "</dd>"),
                      "</dl>")
         }
-        out <- c(out, "<hr/>\n</body></html>")
+        out <- c(out, "<hr/>\n</div></body></html>")
         list(payload = paste(out, collapse="\n"))
     }
 
-    .HTMLusermanuals <- function()
-    {
+    .HTMLusermanuals <- function() {
         pkgs <- unlist(.get_standard_package_names())
 
         out <- HTMLheader("R User Manuals")
@@ -56,17 +56,16 @@ httpd <- function(path, query, ...)
          	out <- c(out, paste0('<h2>Manuals in package', sQuote(pkg),'</h2>'),
          		 makeVignetteTable(cbind(Package=pkg, vinfo[,c("File", "Title", "PDF", "R"), drop = FALSE])))
      	}
-        out <- c(out, "<hr/>\n</body></html>")
+        out <- c(out, "<hr/>\n</div></body></html>")
         list(payload = paste(out, collapse="\n"))
     }
 
-    .HTMLsearch <- function(query)
-    {
+    .HTMLsearch <- function(query) {
     	bool <- function(x) as.logical(as.numeric(x))
         res <- if(identical(names(query), "category")) {
             utils::help.search(keyword = query, verbose = 1L, use_UTF8 = TRUE)
-        } else if(identical(names(query), "results")) {
-            utils:::.hsearch_results()
+        } else if(identical(names(query), c("objects", "port"))) {
+            .httpd_objects(query["port"])
         } else {
             fields <- types <- NULL
             args <- list(pattern = ".")
@@ -158,7 +157,7 @@ httpd <- function(path, query, ...)
                 }
 	    }
         }
-        out <- c(out, "<hr/>\n</body></html>")
+        out <- c(out, "<hr/>\n</div></body></html>")
         list(payload = paste(out, collapse="\n"))
     }
 
@@ -172,9 +171,9 @@ httpd <- function(path, query, ...)
                 "<tr><th style=\"text-align: left\">Concept</th><th>Frequency</th><th>Packages</th><tr>",
                 paste0("<tr><td>",
                        "<a href=\"/doc/html/Search?pattern=",
-                       vapply(reQuote(s), utils::URLencode, "", reserved = TRUE),
+                       utils::URLencode(reQuote(s), reserved = TRUE),
                        "&fields.concept=1&agrep=0\">",
-                       shtmlify(substring(s, 1, 80)),
+                       shtmlify(substr(s, 1L, 80L)),
                        "</a>",
                        "</td><td style=\"text-align: right\">",
                        concepts$Frequency,
@@ -182,7 +181,7 @@ httpd <- function(path, query, ...)
                        concepts$Packages,
                        "</td></tr>"),
                 "</table>",
-                "</body>",
+                "</div></body>",
                 "</html>"))
         list(payload = paste(out, collapse = "\n"))
     }
@@ -201,42 +200,40 @@ httpd <- function(path, query, ...)
                        keywords$Keyword,
                        "</a>",
                        "</td><td>",
-                       shtmlify(substring(keywords$Concept, 1, 80)),
+                       shtmlify(substr(keywords$Concept, 1L, 80L)),
                        "</td><td style=\"text-align: right\">",
                        keywords$Frequency,
                        "</td><td style=\"text-align: right\">",
                        keywords$Packages,
                        "</td></tr>"),
                 "</table>",
-                "</body>",
+                "</div></body>",
                 "</html>"))
         list(payload = paste(out, collapse = "\n"))
     }
 
-    unfix <- function(file)
-    {
+    unfix <- function(file) {
         ## we need to re-fix links altered by fixup.package.URLs
         ## in R < 2.10.0
         fixedfile <- sub("/html/.*", "/fixedHTMLlinks", file)
         if(file.exists(fixedfile)) {
             top <- readLines(fixedfile)
             lines <- readLines(file)
-            lines <- gsub(paste(top, "library", sep="/"),
+            lines <- gsub(paste0(top, "/library"),
                           "../../", lines, fixed = TRUE)
-            lines <- gsub(paste(top, "doc/", sep = "/"),
+            lines <- gsub(paste0(top, "/doc/"),
                           "../../../doc/", lines, fixed = TRUE)
             return(list(payload=paste(lines, collapse="\n")))
         }
         list(file = file)
     }
 
-    mime_type <- function(path)
-    {
+    mime_type <- function(path) {
         ext <- strsplit(path, ".", fixed = TRUE)[[1L]]
         if(n <- length(ext)) ext <- ext[n] else ""
         switch(ext,
                "css" = "text/css",
-               "gif" = "image/gif", # in R2HTML
+               "gif" = "image/gif",     # in R2HTML
                "jpg" = "image/jpeg",
                "png" = "image/png",
                "svg" = "image/svg+xml",
@@ -244,8 +241,8 @@ httpd <- function(path, query, ...)
                "pdf" = "application/pdf",
                "eps" =,
                "ps" = "application/postscript", # in GLMMGibbs, mclust
-               "sgml" = "text/sgml", # in RGtk2
-               "xml" = "text/xml",  # in RCurl
+               "sgml" = "text/sgml",    # in RGtk2
+               "xml" = "text/xml",      # in RCurl
                "text/plain")
     }
 
@@ -265,7 +262,7 @@ httpd <- function(path, query, ...)
 
     error_page <- function(msg)
         list(payload =
-             paste0(HTMLheader("httpd error"), msg, "\n</body></html>"))
+             paste0(HTMLheader("httpd error"), msg, "\n</div></body></html>"))
 
     cssRegexp <- "^/library/([^/]*)/html/R.css$"
     if (grepl("R\\.css$", path) && !grepl(cssRegexp, path))
@@ -274,10 +271,17 @@ httpd <- function(path, query, ...)
     else if(path == "/favicon.ico")
         return(list(file = file.path(R.home("doc"), "html", "favicon.ico")))
     else if(path == "/NEWS")
-         return(list(file = file.path(R.home("doc"), "html", "NEWS.html")))
+         return(list(file = file.path(R.home("doc"), "html", "NEWS.html"),
+                     "content-type" = "text/html"))
     else if(grepl("^/NEWS[.][[:digit:]]$", path))
-    	return(list(file = file.path(R.home("doc"), sub("/", "", path)),
-    	            "content-type" = "text/plain; encoding=utf-8"))
+    	return(list(file = file.path(R.home("doc"), sub("/", "", path, fixed=TRUE)),
+    	            "content-type" = "text/plain; charset=utf-8"))
+    else if((path == "/doc/html/NEWS.html") &&
+            identical(names(query), c("objects", "port"))) {
+        news <- .httpd_objects(query["port"])
+    	formatted <- toHTML(news, title = "R News")
+        return( list(payload = paste(formatted, collapse="\n")) )
+    }
     else if(!grepl("^/(doc|library|session)/", path))
         return(error_page(paste("Only NEWS and URLs under", mono("/doc"),
                                 "and", mono("/library"), "are allowed")))
@@ -308,8 +312,20 @@ httpd <- function(path, query, ...)
     	if (pkg == "NULL") pkg <- NULL  # There were multiple hits in the console
     	topic <- sub(topicRegexp, "\\2", path)
         ## if a package is specified, look there first, then everywhere
-    	if (!is.null(pkg)) # () avoids deparse here
+    	if (!is.null(pkg)) { # () avoids deparse here
     	    file <- utils::help(topic, package = (pkg), help_type = "text")
+            ## Before searching other packages, check if topic.Rd is
+            ## available as a file in the package.
+            if (!length(file) && linksToTopics) {
+                helppath <- system.file("help", package = pkg)
+                if (nzchar(helppath)) {
+                    contents <- readRDS(sub("/help$", "/Meta/Rd.rds", helppath, fixed = FALSE))
+                    helpfiles <- sub("\\.[Rr]d$", "", contents$File)
+                    if (topic %in% helpfiles) file <- file.path(helppath, topic)
+                }
+            }
+        }
+        ## Next, search for topic in all installed packages
     	if (!length(file))
             file <- utils::help(topic, help_type = "text", try.all.packages = TRUE)
 	if (!length(file)) {
@@ -341,23 +357,25 @@ httpd <- function(path, query, ...)
                     "unknown title" else
                     tmp[file_path_sans_ext(tmp$File) == tp[i], "Title"]
             }
-            packages <- paste('<dt><a href="../../', basename(paths), '/html/',
-                              basename(file), '.html">', titles,
-                              '</a></dt><dd> (in package <a href="../../',
-                              basename(paths),
-                              '/html/00Index.html">', basename(paths),
-                              '</a> in library ', dirname(paths), ")</dd>",
-                              sep = "", collapse = "\n")
+            packages <- paste0('<dt><a href="../../',
+                               basename(paths), '/html/',
+                               basename(file), '.html">', titles,
+                               '</a></dt><dd> (in package <a href="../../',
+                               basename(paths),
+                               '/html/00Index.html">', basename(paths),
+                               '</a> in library ', dirname(paths), ")</dd>",
+                               collapse = "\n")
 
             return(list(payload =
-                        paste("<p>",
-                              ## for languages with multiple plurals ....
-                              sprintf(ngettext(length(paths),
-                                               "Help on topic '%s' was found in the following package:",
-                                               "Help on topic '%s' was found in the following packages:"
-                                               ), topic),
-                              "</p><dl>\n",
-                              packages, "</dl>", sep = "", collapse = "\n")
+                        paste0("<p>",
+                               ## for languages with multiple plurals ....
+                               sprintf(ngettext(length(paths),
+                                                "Help on topic '%s' was found in the following package:",
+                                                "Help on topic '%s' was found in the following packages:"
+                                                ), topic),
+                               "</p><dl>\n",
+                               packages, "</dl>",
+                               collapse = "\n")
                         ))
         }
     } else if (grepl(fileRegexp, path)) {
@@ -390,9 +408,9 @@ httpd <- function(path, query, ...)
             return(error_page(msg))
         }
         ## if 'topic' is not a help doc, try it as an alias in the package
-        contents <- readRDS(sub("/help", "/Meta/Rd.rds", path, fixed = TRUE))
+        contents <- readRDS(sub("/help$", "/Meta/Rd.rds", path, fixed = FALSE))
         files <- sub("\\.[Rr]d$", "", contents$File)
-        if(! helpdoc %in% files) {
+        if(helpdoc %notin% files) {
             ## or call help()
             aliases <- contents$Aliases
             lens <- lengths(aliases)
@@ -488,15 +506,24 @@ httpd <- function(path, query, ...)
 				")' in the console.")) )
     } else if (grepl(newsRegexp, path)) {
     	pkg <- sub(newsRegexp, "\\1", path)
-    	if (!is.null(query) && !is.na(subset <- query["subset"])) {
-    	    # See utils:::print.news_db for the encoding of the subset
-    	    rle <- strsplit(subset, "_")[[1]]
-    	    rle <- structure(list(lengths = as.numeric(rle),
-    	    	                  values = rep(c(TRUE, FALSE), length.out = length(rle))),
-    	    	             class = "rle")
-    	    news <- news(inverse.rle(rle)[-1], package = pkg)
-	} else
-    	    news <- news(package = pkg)
+        if(identical(names(query), c("objects", "port")))
+            news <- .httpd_objects(query["port"])
+        else {
+            ## <FIXME>
+            ## This should no longer be used ...
+            if (!is.null(query) && !is.na(subset <- query["subset"])) {
+                ## See utils:::print.news_db for the encoding of the
+                ## subset 
+                rle <- strsplit(subset, "_")[[1L]]
+                rle <- structure(list(lengths = as.numeric(rle),
+                                      values = rep_len(c(TRUE, FALSE),
+                                                       length(rle))),
+                                 class = "rle")
+                news <- news(inverse.rle(rle)[-1L], package = pkg)
+            } else
+                news <- news(package = pkg)
+            ## </FIXME>
+        }
     	formatted <- toHTML(news,
     		            title=paste("NEWS in package", sQuote(pkg)),
     			    up="html/00Index.html")
@@ -559,7 +586,7 @@ httpd <- function(path, query, ...)
         file <- file.path(R.home("doc"), sub("^/doc", "", path))
         if(file.exists(file))
             list(file = file, "content-type" = mime_type(path))
-        else if(file.exists(file <- sub("/manual/", "/html/", file))) {
+        else if(file.exists(file <- sub("/manual/", "/html/", file, fixed=TRUE))) {
             ## tarball has pre-built version of R-admin.html
             list(file = file, "content-type" = mime_type(path))
         } else {
@@ -636,7 +663,7 @@ startDynamicHelp <- function(start = TRUE)
         for(i in seq_along(ports)) {
             ## the next can throw an R-level error,
             ## so do not assign port unless it succeeds.
-	    status <- .Call(startHTTPD, "127.0.0.1", ports[i])
+	    status <- .Call(C_startHTTPD, "127.0.0.1", ports[i])
 	    if (status == 0L) {
                 OK <- TRUE
                 httpdPort(ports[i])
@@ -656,7 +683,7 @@ startDynamicHelp <- function(start = TRUE)
         }
     } else {
         ## Not really tested
-        .Call(stopHTTPD)
+        .Call(C_stopHTTPD)
     	httpdPort(0L)
     }
     invisible(httpdPort())
@@ -669,3 +696,14 @@ function(path, port = httpdPort())
 ## environment holding potential custom httpd handlers
 .httpd.handlers.env <- new.env()
 
+.httpd_objects <-
+local({
+    val <- list()
+    function(port, new) {
+        port <- as.character(port)
+        if(!missing(new))
+            val[[port]] <<- new
+        else
+            val[[port]]
+    }
+})
