@@ -1,7 +1,7 @@
 #  File src/library/utils/R/packages2.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2017 The R Core Team
+#  Copyright (C) 1995-2020 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@ isBasePkg <- function(pkg) {
 
 getDependencies <-
     function(pkgs, dependencies = NA, available = NULL, lib = .libPaths()[1L],
-             binary = FALSE, ...)
+             binary = FALSE, ..., av2 = NULL) ## ... is passed to installed.packages().
 {
     if (is.null(dependencies)) return(unique(pkgs))
     oneLib <- length(lib) == 1L
@@ -51,23 +51,52 @@ getDependencies <-
     }
     p0 <- unique(pkgs)
     miss <-  !p0 %in% row.names(available)
-    if(sum(miss)) {
-        msg <- paste0(if(binary) "as a binary package ", "for ",
-                      sub(" *\\(.*","", R.version.string))
-	warning(sprintf(ngettext(sum(miss),
-				 "package %s is not available (%s)",
-				 "packages %s are not available (%s)"),
-			paste(sQuote(p0[miss]), collapse = ", "), msg),
+    base <- vapply(p0, isBasePkg, FALSE)
+    if (sum(base))
+        warning(sprintf(ngettext(sum(base),
+                                 "package %s is a base package, and should not be updated",
+                                 "packages %s are base packages, and should not be updated"),
+                        paste(sQuote(p0[base]), collapse = ", ")),
                 domain = NA, call. = FALSE)
-        base <- vapply(p0[miss], isBasePkg, FALSE)
-        if (sum(base))
-          warning(sprintf(ngettext(sum(base),
-                                   "package %s is a base package, and should not be updated",
-                                   "packages %s are base packages, and should not be updated"),
-                          paste(sQuote(p0[miss][base]), collapse = ", ")),
-                  domain = NA, call. = FALSE)
-        if (sum(miss) == 1L &&
-            !is.na(w <- match(tolower(p0[miss]),
+    m0 <- miss & !base
+    msg2 <- NULL
+    if(sum(m0) && !is.null(av2)) {
+        keep <- rownames(av2) %in% p0[m0] ## there might be duplicate matches
+        av2 <- av2[keep, , drop = FALSE]
+        if(nrow(av2)) {
+            ds <- av2[, "Depends"]
+            ds[is.na(ds)] <- ""
+            x <- lapply(strsplit(sub("^[[:space:]]*", "", ds),
+                                 "[[:space:]]*,[[:space:]]*"),
+                        function(s) s[grepl("^R[[:space:]]*\\(", s)])
+            lens <- lengths(x)
+            pos <- which(lens > 0L)
+            av2 <- av2[pos,, drop = FALSE]; x <- x[pos]
+            msg2 <- paste(sQuote(av2[, "Package"]), "version", av2[, "Version"],
+                          "is in the repositories but depends on", unlist(x))
+        }
+    }
+    if(sum(m0)) {
+        msg <- paste0(if(binary) "as a binary package ",
+                      "for this version of R")
+        msg3 <- c(paste0(ngettext(sum(m0),
+                                  "A version of this package for your version of R might be available elsewhere,\nsee the ideas at\n",
+                                  "Versions of these packages for your version of R might be available elsewhere,\nsee the ideas at\n"),
+                         ## refer to r-patched for released/patched versions
+                         if (grepl("Under development", R.version.string)) {
+                             "https://cran.r-project.org/doc/manuals/r-devel/R-admin.html#Installing-packages"
+                         } else {
+                             "https://cran.r-project.org/doc/manuals/r-patched/R-admin.html#Installing-packages"
+                         })
+                 )
+	warning(sprintf(ngettext(sum(m0),
+				 "package %s is not available %s",
+				 "packages %s are not available %s"),
+			paste(sQuote(p0[m0]), collapse = ", "),
+                        paste(c(msg, msg2, "", msg3), collapse = "\n")),
+               domain = NA, call. = FALSE)
+        if (sum(m0) == 1L &&
+            !is.na(w <- match(tolower(p0[m0]),
                               tolower(row.names(available))))) {
             warning(sprintf("Perhaps you meant %s ?",
                             sQuote(row.names(available)[w])),
@@ -77,7 +106,7 @@ getDependencies <-
     }
     p0 <- p0[!miss]
 
-    if(depends) { # check for dependencies, recursively
+    if(depends && length(p0)) { # check for dependencies, recursively
         p1 <- p0 # this is ok, as 1 lib only
         ## INSTALL prepends 'lib' to the libpath
         ## Here we are slightly more conservative
@@ -139,6 +168,8 @@ install.packages <-
              keep_outputs = FALSE,
              ...)
 {
+    if (!is.character(type))
+        stop("invalid 'type'; must be a character string")
     type2 <- .Platform$pkgType
     if (type == "binary") {
         if (type2 == "source")
@@ -215,7 +246,7 @@ install.packages <-
             paste(INSTALL_opts[[get_package_name(pkg)]], collapse = " ")
     }
 
-    if(missing(pkgs) || !length(pkgs)) {
+    if(missing(pkgs)) {
         if(!interactive()) stop("no packages were specified")
         ## if no packages were specified, use a menu
 	if(.Platform$OS.type == "windows" || .Platform$GUI == "AQUA"
@@ -242,9 +273,29 @@ install.packages <-
                                 multiple = TRUE,
                                 title = "Packages", graphics = TRUE)
 	}
-	if(!length(pkgs)) stop("no packages were specified")
     }
 
+    if (.Platform$OS.type == "windows" && length(pkgs)) {
+        ## look for package in use.
+        pkgnames <- get_package_name(pkgs)
+        ## there is no guarantee we have got the package name right:
+        ## foo.zip might contain package bar or Foo or FOO or ....
+        ## but we can't tell without trying to unpack it.
+        inuse <- search()
+        inuse <- sub("^package:", "", inuse[grep("^package:", inuse)])
+        inuse <- pkgnames %in% inuse
+        if(any(inuse)) {
+            warning(sprintf(ngettext(sum(inuse),
+                    "package %s is in use and will not be installed",
+                    "packages %s are in use and will not be installed"),
+                            paste(sQuote(pkgnames[inuse]), collapse=", ")),
+                    call. = FALSE, domain = NA, immediate. = TRUE)
+            pkgs <- pkgs[!inuse]
+        }
+    }
+
+    if(!length(pkgs)) return(invisible())
+
     if(missing(lib) || is.null(lib)) {
         lib <- .libPaths()[1L]
 	if(!quiet && length(.libPaths()) > 1L)
@@ -300,18 +351,18 @@ install.packages <-
     ## check if we should infer repos = NULL
     if(length(pkgs) == 1L && missing(repos) && missing(contriburl)) {
         if((type == "source" && any(grepl("[.]tar[.](gz|bz2|xz)$", pkgs))) ||
-           (type %in% "win.binary" && length(grep("[.]zip$", pkgs))) ||
-           (substr(type, 1L, 10L) == "mac.binary" && grepl("[.]tgz$", pkgs))) {
+           (type %in% "win.binary" && endsWith(pkgs, ".zip")) ||
+           (startsWith(type, "mac.binary") && endsWith(pkgs, ".tgz"))) {
             repos <- NULL
             message("inferring 'repos = NULL' from 'pkgs'")
         }
         if (type == "both") {
-            if (type2 %in% "win.binary" && grepl("[.]zip$", pkgs)) {
+            if (type2 %in% "win.binary" && endsWith(pkgs, ".zip")) {
                 repos <- NULL
                 type <- type2
                 message("inferring 'repos = NULL' from 'pkgs'")
-            } else if (substr(type2, 1L, 10L) == "mac.binary"
-                       && grepl("[.]tgz$", pkgs)) {
+            } else if (startsWith(type2, "mac.binary")
+                       && endsWith(pkgs, ".tgz")) {
                 repos <- NULL
                 type <- type2
                 message("inferring 'repos = NULL' from 'pkgs'")
@@ -325,9 +376,9 @@ install.packages <-
 
     ## check if we should infer the type
     if (length(pkgs) == 1L && is.null(repos) && type == "both") {
-    	if (  (type2 %in% "win.binary" && grepl("[.]zip$", pkgs))
-	    ||(substr(type2, 1L, 10L) == "mac.binary"
-		   && grepl("[.]tgz$", pkgs))) {
+    	if (  (type2 %in% "win.binary" && endsWith(pkgs, ".zip"))
+	    ||(startsWith(type2, "mac.binary")
+		   && endsWith(pkgs, ".tgz"))) {
 	    type <- type2
 	} else if (grepl("[.]tar[.](gz|bz2|xz)$", pkgs)) {
 	    type <- "source"
@@ -393,7 +444,22 @@ install.packages <-
         bins <- pkgs[pkgs %in% bins]
         srcOnly <- pkgs[! pkgs %in% bins]
         binvers <- av2[bins, "Version"]
-        hasSrc <-  !is.na(av2[bins, "Archs"])
+
+        ## In most cases, packages that need compilation have non-NA "Archs"
+        ## in their binary version and "NeedsCompilation" with value "yes"
+        ## in their source version.  However, the fields are not always
+        ## filled in correctly and some binary packages have executable code
+        ## outside "libs" (so "Archs" is NA), also a later version of a
+        ## package may need compilation but an older one not.  To reduce the
+        ## risk that the user will attempt to install a package from source
+        ## but without having the necessary tools to build it, packages are
+        ## treated as needing compilation whenever they have non-NA "Archs"
+        ## in binary version or/and "NeedsCompilation"="yes" in source
+        ## version.
+
+        hasArchs <-  !is.na(av2[bins, "Archs"])
+        needsCmp <- !(available[bins, "NeedsCompilation"] %in% "no")
+        hasSrc <- hasArchs | needsCmp
 
         srcvers <- available[bins, "Version"]
         later <- as.numeric_version(binvers) < srcvers
@@ -479,7 +545,7 @@ install.packages <-
 	flush.console()
         ## end of "both"
     } else if (getOption("install.packages.check.source", "yes") %in% "yes"
-               && (type %in% "win.binary" || substr(type, 1L, 10L) == "mac.binary")) {
+               && (type %in% "win.binary" || startsWith(type, "mac.binary"))) {
         if (missing(contriburl) && is.null(available) && !is.null(repos)) {
             contriburl2 <- contrib.url(repos, "source")
 	    # The line above may have changed the repos option, so..
@@ -533,7 +599,7 @@ install.packages <-
     }
 
     if(.Platform$OS.type == "windows") {
-        if(substr(type, 1L, 10L) == "mac.binary")
+        if(startsWith(type, "mac.binary"))
             stop("cannot install macOS binary packages on Windows")
 
         if(type %in% "win.binary") {
@@ -556,9 +622,9 @@ install.packages <-
         }
         ## Avoid problems with backslashes
         ## -- will mess up UNC names, but they don't work
-        pkgs <- gsub("\\\\", "/", pkgs)
+        pkgs <- gsub("\\", "/", pkgs, fixed=TRUE)
     } else {
-        if(substr(type, 1L, 10L) == "mac.binary") {
+        if(startsWith(type, "mac.binary")) {
             if(!grepl("darwin", R.version$platform))
                 stop("cannot install macOS binary packages on this platform")
             .install.macbinary(pkgs = pkgs, lib = lib, contriburl = contriburl,
@@ -575,15 +641,6 @@ install.packages <-
             stop("This version of R is not set up to install source packages\nIf it was installed from an RPM, you may need the R-devel RPM")
     }
 
-    ## we need to ensure that R CMD INSTALL runs with the same
-    ## library trees as this session.
-    ## FIXME: At least on Windows, either run sub-R directly (to avoid sh)
-    ## or run the install in the current process.
-    libpath <- .libPaths()
-    libpath <- libpath[! libpath %in% .Library]
-    if(length(libpath))
-        libpath <- paste(libpath, collapse = .Platform$path.sep)
-
     cmd0 <- file.path(R.home("bin"), "R")
     args0 <- c("CMD", "INSTALL")
 
@@ -609,7 +666,11 @@ install.packages <-
         stop(gettextf("invalid %s argument", sQuote("keep_outputs")),
              domain = NA)
 
-    if(length(libpath)) {
+    ## we need to ensure that R CMD INSTALL runs with the same
+    ## library trees, i.e., .R_LIBS() as this session.
+    ## FIXME: At least on Windows, either run sub-R directly (to avoid sh)
+    ## or run the install in the current process.
+    if(length(libpath <- .R_LIBS())) {
         ## <NOTE>
         ## For the foreseeable future, the 'env' argument to system2()
         ## on Windows is limited to calls to make and rterm (but not R
@@ -643,7 +704,7 @@ install.packages <-
                          paste(c(cmd0, args0), collapse = " ")),
                 domain = NA)
 
-    if(is.null(repos) & missing(contriburl)) {
+    if(is.null(repos) && missing(contriburl)) {
         ## install from local source tarball(s)
         update <- cbind(path.expand(pkgs), lib) # for side-effect of recycling to same length
 
@@ -651,7 +712,7 @@ install.packages <-
            if (is.na(update[i, 1L])) next
            args <- c(args0,
                      get_install_opts(update[i, 1L]),
-                     "-l", shQuote(update[i, 2L]),
+                     "-l",    shQuote(update[i, 2L]),
                      getConfigureArgs(update[i, 1L]),
                      getConfigureVars(update[i, 1L]),
                      shQuote(update[i, 1L]))
@@ -672,7 +733,7 @@ install.packages <-
     }
 
     tmpd <- destdir
-    nonlocalrepos <- length(grep("^file:", contriburl)) < length(contriburl)
+    nonlocalrepos <- !all(startsWith(contriburl, "file:"))
     if(is.null(destdir) && nonlocalrepos) {
         tmpd <- file.path(tempdir(), "downloaded_packages")
         if (!file.exists(tmpd) && !dir.create(tmpd))
@@ -681,11 +742,26 @@ install.packages <-
                  domain = NA)
     }
 
-    if(is.null(available))
-        available <- available.packages(contriburl = contriburl,
-                                        method = method, ...)
-    if(getDeps)
-        pkgs <- getDependencies(pkgs, dependencies, available, lib, ...)
+    ## from here on we deal with source packages in repos
+    av2 <- NULL
+    if(is.null(available)) {
+        filters <- getOption("available_packages_filters")
+        if(!is.null(filters)) {
+            available <- available.packages(contriburl = contriburl,
+                                            method = method, ...)
+        } else {
+            f <- setdiff(available_packages_filters_default,
+                         c("R_version", "duplicates"))
+            av2 <- available.packages(contriburl = contriburl, filters = f,
+                                      method = method, ...)
+            f <- available_packages_filters_db[["R_version"]]
+            f2 <- available_packages_filters_db[["duplicates"]]
+            available <- f2(f(av2))
+        }
+    }
+    if(getDeps) ## true except for type = "both" above.
+        pkgs <- getDependencies(pkgs, dependencies, available, lib, ...,
+                                av2 = av2)
 
     foundpkgs <- download.packages(pkgs, destdir = tmpd, available = available,
                                    contriburl = contriburl, method = method,
@@ -725,12 +801,12 @@ install.packages <-
             ## if --no-lock or --lock was specified in INSTALL_opts
             ## that will override this.
             args0 <- c(args0, "--pkglock")
-            tmpd <- file.path(tempdir(), "make_packages")
-            if (!file.exists(tmpd) && !dir.create(tmpd))
+            tmpd2 <- file.path(tempdir(), "make_packages")
+            if (!file.exists(tmpd2) && !dir.create(tmpd2))
                 stop(gettextf("unable to create temporary directory %s",
-                              sQuote(tmpd)),
+                              sQuote(tmpd2)),
                      domain = NA)
-            mfile <- file.path(tmpd, "Makefile")
+            mfile <- file.path(tmpd2, "Makefile")
             conn <- file(mfile, "wt")
             deps <- paste(paste0(update[, 1L], ".ts"), collapse=" ")
             deps <- strwrap(deps, width = 75, exdent = 2)
@@ -739,12 +815,13 @@ install.packages <-
             aDL <- .make_dependency_list(upkgs, available, recursive = TRUE)
             for(i in seq_len(nrow(update))) {
                 pkg <- update[i, 1L]
+                fil <- update[i, 3L]
                 args <- c(args0,
-                          get_install_opts(update[i, 3L]),
+                          get_install_opts(fil),
                           "-l", shQuote(update[i, 2L]),
-                          getConfigureArgs(update[i, 3L]),
-                          getConfigureVars(update[i, 3L]),
-                          shQuote(update[i, 3L]),
+                          getConfigureArgs(fil),
+                          getConfigureVars(fil),
+                          shQuote(fil),
                           ">", paste0(pkg, ".out"),
                           "2>&1")
                 ## <NOTE>
@@ -774,7 +851,7 @@ install.packages <-
                     "", sep = "\n", file = conn)
             }
             close(conn)
-            cwd <- setwd(tmpd)
+            cwd <- setwd(tmpd2)
             on.exit(setwd(cwd))
             ## MAKE will be set by sourcing Renviron
             status <- system2(Sys.getenv("MAKE", "make"),
@@ -793,18 +870,28 @@ install.packages <-
             }
             if(keep_outputs)
                 file.copy(paste0(update[, 1L], ".out"), outdir)
+            ## Keep binary packages possibly created via --build
+            file.copy(Sys.glob(paste0(update[, 1L], "*.zip")), cwd)
+            file.copy(Sys.glob(paste0(update[, 1L], "*.tgz")), cwd)
+            file.copy(Sys.glob(paste0(update[, 1L], "*.tar.gz")), cwd)
             setwd(cwd); on.exit()
-            unlink(tmpd, recursive = TRUE)
+            unlink(tmpd2, recursive = TRUE)
         } else {
-            outfiles <- paste0(update[, 1L], ".out")
+            tmpd2 <- tempfile()
+            if(!dir.create(tmpd2))
+                stop(gettextf("unable to create temporary directory %s",
+                              sQuote(tmpd2)),
+                     domain = NA)
+            outfiles <- file.path(tmpd2, paste0(update[, 1L], ".out"))
             for(i in seq_len(nrow(update))) {
                 outfile <- if(keep_outputs) outfiles[i] else output
+                fil <- update[i, 3L]
                 args <- c(args0,
-                          get_install_opts(update[i, 3L]),
+                          get_install_opts(fil),
                           "-l", shQuote(update[i, 2L]),
-                          getConfigureArgs(update[i, 3L]),
-                          getConfigureVars(update[i, 3L]),
-                          update[i, 3L])
+                          getConfigureArgs(fil),
+                          getConfigureVars(fil),
+                          shQuote(fil))
                 status <- system2(cmd0, args, env = env,
                                   stdout = outfile, stderr = outfile,
                                   timeout = tlim)
@@ -821,10 +908,9 @@ install.packages <-
                             domain = NA)
                 }
             }
-            if(keep_outputs && (outdir != getwd())) {
+            if(keep_outputs)
                 file.copy(outfiles, outdir)
-                file.remove(outfiles)
-            }
+            unlink(tmpd2, recursive = TRUE)
         }
         ## Using stderr is the wish of PR#16420
         if(!quiet && nonlocalrepos && !is.null(tmpd) && is.null(destdir))
@@ -840,7 +926,7 @@ install.packages <-
     } else if(!is.null(tmpd) && is.null(destdir)) unlink(tmpd, TRUE)
 
     invisible()
-}
+}##end install.packages
 
 ## treat variables as global in a package, for codetools & check
 globalVariables <- function(names, package, add = TRUE)
@@ -893,3 +979,13 @@ packageName <- function(env = parent.frame()) {
 	"base"
     ## else NULL
 }
+
+##' R's .libPaths() to be used in 'R CMD ...' or similar,
+##' most easily by a previous  Sys.setenv(R_LIBS = .R_LIBS())
+## not yet exported
+.R_LIBS <- function(libp = .libPaths()) {
+    libp <- libp[! libp %in% .Library]
+    if(length(libp))
+        paste(libp, collapse = .Platform$path.sep)
+    else "" # character(0) would fail in Sys.setenv()
+}
