@@ -35,7 +35,7 @@ static R_InternetRoutines routines, *ptr = &routines;
 /*
 SEXP Rdownload(SEXP args);
 Rconnection R_newurl(char *description, char *mode);
-Rconnection R_newsock(char *host, int port, int server, int serverfd, char *mode, int timeout);
+Rconnection R_newsock(char *host, int port, int server, int serverfd, char *mode, int timeout, int options);
 Rconnection R_newservsock(int port);
 
 
@@ -104,11 +104,11 @@ R_newurl(const char *description, const char * const mode, SEXP headers, int typ
 
 Rconnection attribute_hidden
 R_newsock(const char *host, int port, int server, int serverfd,
-          const char * const mode, int timeout)
+          const char * const mode, int timeout, int options)
 {
     if(!initialized) internet_Init();
     if(initialized > 0)
-	return (*ptr->newsock)(host, port, server, serverfd, mode, timeout);
+	return (*ptr->newsock)(host, port, server, serverfd, mode, timeout, options);
     else {
 	error(_("internet routines cannot be loaded"));
 	return (Rconnection)0;
@@ -226,8 +226,13 @@ SEXP Rsockconnect(SEXP sport, SEXP shost)
 SEXP Rsockread(SEXP ssock, SEXP smaxlen)
 {
     if (length(ssock) != 1) error("invalid 'socket' argument");
-    int sock = asInteger(ssock), maxlen = asInteger(smaxlen);
-    char buf[maxlen+1], *abuf[1];
+    int sock = asInteger(ssock);
+    int maxlen = asInteger(smaxlen);
+    if (maxlen < 0) /* also catches NA_INTEGER */
+	error(_("maxlen must be non-negative"));
+    SEXP rbuf = allocVector(RAWSXP, maxlen + 1);
+    PROTECT(rbuf);
+    char *buf = (char *) RAW(rbuf), *abuf[1];
     abuf[0] = buf;
     if(!initialized) internet_Init();
     if(initialized > 0)
@@ -238,7 +243,7 @@ SEXP Rsockread(SEXP ssock, SEXP smaxlen)
 	error("Error reading data in Rsockread");
     SEXP ans = PROTECT(allocVector(STRSXP, 1));
     SET_STRING_ELT(ans, 0, mkCharLen(buf, maxlen));
-    UNPROTECT(1);
+    UNPROTECT(2); /* rbuf, ans */
     return ans;
 		       
 }
@@ -362,6 +367,6 @@ R_newCurlUrl(const char *description, const char * const mode, SEXP headers, int
 	error(_("internet routines cannot be loaded"));
 	return (Rconnection)0;
     }
-    return (Rconnection)0; /* -Wall */
+    return (Rconnection)0; /* -Wall in gcc, but Solaris compiler complains */
 }
 
