@@ -927,13 +927,17 @@ in version 1 workspaces"));
 	/* FALLTHROUGH */
     case LISTSXP:
     case LANGSXP:
-    case CLOSXP:
     case PROMSXP:
     case DOTSXP:
 	NewMakeLists(TAG(obj), sym_list, env_list);
 	NewMakeLists(CAR(obj), sym_list, env_list);
 	NewMakeLists(CDR(obj), sym_list, env_list);
 	break;
+    case CLOSXP:
+	NewMakeLists(CLOENV(obj), sym_list, env_list);
+	NewMakeLists(FORMALS(obj), sym_list, env_list);
+	NewMakeLists(BODY(obj), sym_list, env_list);
+	break;
     case EXTPTRSXP:
 	NewMakeLists(EXTPTR_PROT(obj), sym_list, env_list);
 	NewMakeLists(EXTPTR_TAG(obj), sym_list, env_list);
@@ -1054,7 +1058,6 @@ static void NewWriteItem (SEXP s, SEXP sym_list, SEXP env_list, FILE *fp, Output
 	    break;
 	case LISTSXP:
 	case LANGSXP:
-	case CLOSXP:
 	case PROMSXP:
 	case DOTSXP:
 	    /* Dotted pair objects */
@@ -1062,6 +1065,11 @@ static void NewWriteItem (SEXP s, SEXP sym_list, SEXP env_list, FILE *fp, Output
 	    NewWriteItem(CAR(s), sym_list, env_list, fp, m, d);
 	    NewWriteItem(CDR(s), sym_list, env_list, fp, m, d);
 	    break;
+	case CLOSXP:
+	    NewWriteItem(CLOENV(s), sym_list, env_list, fp, m, d);
+	    NewWriteItem(FORMALS(s), sym_list, env_list, fp, m, d);
+	    NewWriteItem(BODY(s), sym_list, env_list, fp, m, d);
+	    break;
 	case EXTPTRSXP:
 	    NewWriteItem(EXTPTR_PROT(s), sym_list, env_list, fp, m, d);
 	    NewWriteItem(EXTPTR_TAG(s), sym_list, env_list, fp, m, d);
@@ -1143,7 +1151,7 @@ static void NewDataSave (SEXP s, FILE *fp, OutputRoutines *m, SaveLoadData *d)
 	R_assert(TYPEOF(CAR(iterator)) == ENVSXP);
 	NewWriteItem(ENCLOS(CAR(iterator)), sym_table, env_table, fp, m, d);
 	NewWriteItem(FRAME(CAR(iterator)), sym_table, env_table, fp, m, d);
-	NewWriteItem(TAG(CAR(iterator)), sym_table, env_table, fp, m, d);
+	NewWriteItem(HASHTAB(CAR(iterator)), sym_table, env_table, fp, m, d);
     }
     NewWriteItem(s, sym_table, env_table, fp, m, d);
 
@@ -2477,9 +2485,12 @@ SEXP attribute_hidden do_loadFromConn2(SEXP call, SEXP op, SEXP args, SEXP env)
 	strncmp((char*)buf, "RDX3\n", 5) == 0) {
 	R_InitConnInPStream(&in, con, R_pstream_any_format, NULL, NULL);
 	if (PRIMVAL(op) == 0) {
+	    int old_InitReadItemDepth = R_InitReadItemDepth,
+		old_ReadItemDepth = R_ReadItemDepth;
 	    R_InitReadItemDepth = R_ReadItemDepth = -asInteger(CADDR(args));
 	    res = RestoreToEnv(R_Unserialize(&in), aenv);
-	    R_ReadItemDepth = 0;
+	    R_InitReadItemDepth = old_InitReadItemDepth;
+	    R_ReadItemDepth = old_ReadItemDepth;
 	} else 
 	    res = R_SerializeInfo(&in);
 	if(!wasopen) {
