@@ -33,15 +33,6 @@
 #include "sock.h"
 #include <errno.h>
 
-#ifdef Win32
-# ifndef EINTR
-#  define EINTR                   WSAEINTR
-# endif
-# ifndef EWOULDBLOCK
-#  define EWOULDBLOCK             WSAEWOULDBLOCK
-# endif
-#endif
-
 static void listencleanup(void *data)
 {
     int *psock = data;
@@ -105,6 +96,9 @@ static Rboolean sock_open(Rconnection con)
     }
     this->fd = sock;
 
+    if (this->options & RSC_SET_TCP_NODELAY)
+	R_set_nodelay(sock);
+
     mlen = (int) strlen(con->mode);
     con->isopen = TRUE;
     if(mlen >= 2 && con->mode[mlen - 1] == 'b') con->text = FALSE;
@@ -142,10 +136,11 @@ static ssize_t sock_read_helper(Rconnection con, void *ptr, size_t size)
 	    do
 		res = R_SockRead(this->fd, this->inbuf, 4096, 
 				 con->blocking, this->timeout);
-	    while (-res == EINTR);
 #ifdef Win32
-	    if (! con->blocking && -res == EAGAIN) {
+	    while (-res == WSAEINTR);
+	    if (! con->blocking && -res == WSAEWOULDBLOCK) {
 #else
+	    while (-res == EINTR);
 	    if (! con->blocking && (-res == EAGAIN || -res == EWOULDBLOCK)) {
 #endif
 		con->incomplete = TRUE;
@@ -185,7 +180,7 @@ static int sock_fgetc_internal(Rconnection con)
 static size_t sock_read(void *ptr, size_t size, size_t nitems,
 			Rconnection con)
 {
-    ssize_t n = sock_read_helper(con, ptr, size * nitems)/size;
+    ssize_t n = sock_read_helper(con, ptr, size * nitems)/((ssize_t)size);
     return n > 0 ? n : 0;
 }
 
@@ -194,12 +189,12 @@ static size_t sock_write(const void *ptr, size_t size, size_t nitems,
 {
     Rsockconn this = (Rsockconn)con->private;
     ssize_t n = R_SockWrite(this->fd, ptr, (int)(size * nitems),
-			    this->timeout)/size;
+			    this->timeout)/((ssize_t)size);
     return n > 0 ? n : 0;
 }
 
 Rconnection in_R_newsock(const char *host, int port, int server, int serverfd,
-			 const char * const mode, int timeout)
+			 const char * const mode, int timeout, int options)
 {
     Rconnection new;
 
@@ -236,6 +231,7 @@ Rconnection in_R_newsock(const char *host, int port, int server, int serverfd,
     ((Rsockconn)new->private)-> server = server;
     ((Rsockconn)new->private)-> timeout = timeout;
     ((Rsockconn)new->private)-> serverfd = serverfd;
+    ((Rsockconn)new->private)-> options = options;
     return new;
 }
 
